(*
 * gcmap.mli
 * 
 * This file is part of the Oxford Oberon-2 compiler
 * Copyright (c) 2006--2016 J. M. Spivey
 * All rights reserved
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *)

type gcmap

(* null_map -- map with no pointers *)
val null_map : gcmap

(* ptr_map -- map for a single pointer *)
val ptr_map : gcmap

(* shift -- shift a GC map by a specified offset *)
val shift : int -> gcmap -> gcmap

(* repeat -- repeat a GC map with specified count and stride *)
val repeat : int -> int -> gcmap -> gcmap

(* join -- union of two maps *)
val join : gcmap -> gcmap -> gcmap

(* union -- union of a list of maps *)
val union : gcmap list -> gcmap

(* flex_map -- map for an open array parameter *)
val flex_map : int -> int -> int -> gcmap -> gcmap

(* make_bitmap -- render a GC map as a bitmap or raise Not_found *)
val make_bitmap : int -> gcmap -> int32

(* put_varmap -- output GC map for a variable *)
val put_varmap : Symtab.symbol -> gcmap -> unit

(* save_map -- name a gcmap with a symbol *)
val save_map : Symtab.symbol -> gcmap -> unit

(* put_maps -- output all named maps *)
val put_maps : unit -> unit
